# -*- coding: utf-8 -*-
"""GuardianExtended.ipynb

Automatically generated by Colaboratory.

Original file is located at
    [OMITTED]

# Steps to get collect the data and process it using the provided URLs
1.   Register on the [Guardians API](https://open-platform.theguardian.com/access/) to get a developer key. The free version is good enough.
3.   Add your developer code to `my_api` variable below. 
2.   Place the code in the same folder as the Four .csv files. 
4. Follow the instructions below, run a cell by cell.
"""

import os, sys
import requests
import time
import re

full_path = "."
my_api = '*****-******' # get an API code from the Guardian API

"""*   The function below retrieves one article at a time, by its URL (which is also its unique ID). Using the API's documentation, we formulated our query to retireve only the `body` content of the article's web page. 
*   Other meta data included with the articles request is the contributor's profile, which contains the author name.
"""

def get_articles(topic):
    path = os.path.join(full_path, topic)

    if not os.path.exists(path):
        os.makedirs(path)

    print(path)

    prev_author = ''
    with open(path+'.csv') as listFile:
        listFile.readline()  # skip the header
        for line in listFile.readlines():
            print(line.split(','))
            author_id, article_id, comment, _ = line.split(',') 
            
            if prev_author != author_id:
                counter = 0
                prev_author = author_id
                if not os.path.exists(os.path.join(path, author_id)):
                    os.makedirs(os.path.join(path, author_id))

            if article_id.strip() == '':
                counter += 1
                continue

            base_url = 'https://content.guardianapis.com/{}?show-tags=contributor&show-fields=body&api-key={}'
            full_url = base_url.format(article_id, my_api)
            
            response = requests.get(full_url)
            if response.status_code == 200:
                # print('Success!')
                jasonized_response = response.json()

                assert author_id == jasonized_response["response"]['content']['tags'][0]['id'].replace('profile/','')
                article_body = jasonized_response["response"]['content']['fields']['body']
                counter += 1

                with open(os.path.join(path, author_id, str(counter)+'.xml'), mode='w', encoding='utf8') as outFile:
                    outFile.write(article_body)
                
            elif response.status_code == 404:
                print('Not Found.')
                break
        

    return

confirm = "no" #replaec no with yes to confirm

if confirm == "no":
	Topics = ['Politics', 'Society', 'UK', 'World']
	for topic in sorted(Topics):
		get_articles(topic) 
else:
	print("You should have already finished this part. Other wise, set confirrm to no")
# you should see the names of the authors, with a URL to the article printed one by one

"""At this point, you should have four new folders corresponding to the topics. Inside each folder there will be 13 subfolders, one for each author. Finally, each author folder will contain their articles, in raw html up to 10 articles per author

## OPTIONAL: Viewing the html tags in the articles.
"""

from html.parser import HTMLParser
import urllib.request as urllib2

# Optional class to show/print html tags in text
class MyHTMLParser(HTMLParser):
   #Initializing lists
   lsStartTags = list()
   lsEndTags = list()
   lsStartEndTags = list()
   lsComments = list()

   #HTML Parser Methods
   def handle_starttag(self, startTag, attrs):
       if startTag not in ['p', 'br', 'strong', 'em','b', 'i', 'figcaption', 'blockquote', 'ul']:
           print(startTag, attrs)
       print(startTag, attrs)
       self.lsStartTags.append(startTag)

   def handle_endtag(self, endTag):
    #    print(endTag)
       self.lsEndTags.append(endTag)

   def handle_startendtag(self,startendTag, attrs):
       self.lsStartEndTags.append(startendTag)

   def handle_comment(self,data):
       self.lsComments.append(data)

"""Example on using the class above to view the content of the files."""

# ### ------------ 
# #creating an object of the overridden class
# parser = MyHTMLParser()

# #Opening NYTimes site using urllib2
# html_page = html_page = urllib2.urlopen("https://www.nytimes.com/")

# #Feeding the content
# dm = """<audio controls>
#   <source src="horse.ogg" type="audio/ogg">
#   <source src="horse.mp3" type="audio/mpeg">
#   Your browser does not support the audio element.
# </audio>"""
# parser.feed(dm)

"""## Manual edits to the collected xml files.

The following changes should be done manually before preprocessing the xml files. They are a collection of blurbs that are:


*   either too few to write a code to preprocess them
*   noisey (i.e. missed by the script because they dont follow the HTML rules.
* or hyperlinks (where we kept the original text of a hyperlink, but removed text and hyperlinks for others. E.g. websites of the authors.

### Changes to do manually:



*   Removing "This correction appeared ... " from these only three files:


1.   Politics catherinebennett 8.txt (The first paragraph) `[<b>The following correction was ... succeeded by Hu Jintao in November 2002. ]`
2.   Society royhattersley 8.txt     (The last paragraph)  `[<p><strong>The following correction was ... Stanley Road, Liverpool 5.</p>]`
3. UK martinkettle 6.txt (The last paragraph)            `[<p><b>The following correction was ... is roughly twice that.</p>]`

*   Removing extra html artifacts in the `<body>` of the pages 

1. Politics martinkettle 7.txt : removing email from  `[<p>&lt; A HREF="mailto:martin.kettle@theguardian.com"&gt;martin.kettle@theguardian.com <!-- A --></p>]`
-----
2. Society catherinebennett 3.xml : `[<p><strong>Statements</strong></p> .... <EOF>]`
3. Society pollytoynbee 9.xml : `[<a href="http://www.do-it.org.uk" target_new">] `
4. Society pollytoynbee 9.xml :` <a href="http://www.csv.org.uk" target_new">]`
-------------
5. UK pollytoynbee 4.xml : `[<p><b>Related stories - news</b> .... <EOF>]`
6. UK nickcohen 6.xml : `[<p><b>Email update</b> .... <EOF>] `
7. UK nickcohen 10.xml : `[<p><b>Interactive guide</b><br> .... <EOF>] `
8. UK hugoyoung 3.xml : `[<p><b>Related articles</b><br> .... <EOF>] `
9. UK jonathanfreedland 8.xml : `[<p><b>Related stories - news</b><br> .... <EOF>]`
10. UK jonathanfreedland 9.xml : `[<p><b>Related stories - news</b><br> .... <EOF>]`
11. UK martinkettle 3.xml : `[ <a href="martin.kettle@theguardian.com">martin.kettle@theguardian.com</a>]`
12. UK peterpreston 2.xml : `[<p><a href="http://www.p.preston@theguardian.com">p.preston@theguardian.com</a></p> ]`
13. UK peterpreston 7.xml : `[<p><br><a href="nailto:peter.preston@theguardian.com">Peter Preston</a></p>]`
14. UK royhattersley 10.xml : `[<p> <a href="comment@theguardian.com">comment@theguardian.com</a></p>]`
---------
15. World jonathanfreedland 2.xml : `[<p><strong>·</strong> This article was amended on Thursday February 12 2009.</p>]`
16. World jonathanfreedland 8.xml : `[<p><strong>·</strong> You can hear Jonathan Freedland ....  <EOF>]`
17. World maryriddell 1.xml : `[<p><a href="maiolto:mary ....  <EOF>]`
18. World royhattersley 8.xml : `[<p> <a href="maitlo:comment@theguardian.com">comment@ ....  <EOF>`
"""

#Make the previous changes, then change 


tags = []
if confirm == "yes":
    for topic in sorted(os.listdir(full_path)) :

        path = os.path.join(full_path, topic)
        if os.path.isdir(path):
            for author in sorted(os.listdir(path)):
                subpath = os.path.join(path, author)
                for filename in sorted(os.listdir(subpath)):
                    fpath = os.path.join(subpath, filename)
                    if sys.version_info < (3,):
                        f = open(fpath)
                    else:
                        f = open(fpath, encoding='latin-1')
                    t = f.read()

                    ## to use the class above for view, add this before and after the preprocessing steps
                    # parser.feed(t)

                    print('=============================', topic, author, filename)
                    # remove EMAIL hyperlink tags, and the linked text (including everything between the tags)
                    t = re.sub(re.compile("(<p>)?<a\s+(?:[^>]*?\s+)?href=\"mailto(.)*>(.)*</a>(</p>)?"), '', t)

                    # remov mnobiot's website signature (including everything between the tags)
                    t = re.sub(re.compile("(<p>)?<a\s+(?:[^>]*?\s+)?href=\"(.)*monbiot(.)*>(.)*</a>(</p>)?"), '', t)

                    # REMOVEW VIDEOS (including everything between the tags)
                    t = re.sub(re.compile("<video\s+data-media-id(.)*>(.)*</video>"), '', t)

                    # Remove figures (including everything between the tags)
                    t = re.sub(re.compile("<figure class=(.[^>])*>(.)*</figure>"), '', t)

                    # remove Spans
                    t = re.sub(re.compile("</?span(\s+class=\"(.)*\")?>"), '', t)

                    # remove ASIDE
                    t = re.sub(re.compile("</?aside(\s+class=\"(.)*\")?>"), '', t)

                    # remove generic hyperlink tags, but keep the content 
                    t = re.sub(re.compile("<a\s+(?:[^>]*?\s+)?href=\"([^\"]*)\"(\s*title=\"([^\"]*)\")?(\s+target=\"([^\"]*)\")?>"), '', t)
                    
                    # Finally, remove formatting tags, and closing tags (if any)
                    t = re.sub(re.compile("</?[apbi]?(br)?(ul)?(strong)?(em)?>"), '', t)

                    # printing after preprocessing to make sure the text is clean 
                    # parser.feed(t)

                    # We used xml because it is easier to see the tags
                    with open(fpath.replace('.xml', '.txt'), mode='w', encoding='latin-1') as newXML:
                        newXML.write(t) 

                # break # processes one author in the current topic
           # break # processes one topic out of the four
    else:
        print("You did not confirm doing the manual changes!")

"""Finally, clean up all the xml files, and keep only the text files."""

for topic in sorted(os.listdir(full_path)) :

    path = os.path.join(full_path, topic)
    if os.path.isdir(path):
        for author in sorted(os.listdir(path)):
            subpath = os.path.join(path, author)
            for filename in sorted(os.listdir(subpath)):
                fpath = os.path.join(subpath, filename)
                fpath.replace('.txt', '.xml')
                if os.path.isfile(fpath) and fpath.endswith('.xml'):
                    os.remove(fpath)  

        #     break # processes one author in the current topic
        # break # processes one topic out of the four

